/*
 * PmodGPS.c
 *
 *  Created on: Nov 20, 2017
 *      Author: Krzysztof Chojnowski
 */

#include <string.h>

#include "PmodGPS.h"

// The handle to the UART port used for the communication with the PmodGPS module.
UART_HandleTypeDef pmodGpsUart;

void PmodGPS_Config(void)
{
  // Enable clock for GPIOD port required for 3DF and 1PPS pins.
  __HAL_RCC_GPIOD_CLK_ENABLE();

  // Configure the interrupts on PD10 (1PPS) and PD6 (3DF). The interrupts are active on both edges
  // to indicate the connection and disconnection events.
  GPIO_InitTypeDef GPIO_InitStruct;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING_FALLING;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Pin = GPIO_PIN_6 | GPIO_PIN_10;
  HAL_GPIO_Init(GPIOD, &GPIO_InitStruct);

  HAL_NVIC_SetPriority(EXTI9_5_IRQn, 2, 0);
  HAL_NVIC_SetPriority(EXTI15_10_IRQn, 2, 1);
  HAL_NVIC_EnableIRQ(EXTI9_5_IRQn);
  HAL_NVIC_EnableIRQ(EXTI15_10_IRQn);

  pmodGpsUart.Instance = USART3;
  pmodGpsUart.Init.BaudRate = 9600;
  pmodGpsUart.Init.WordLength = UART_WORDLENGTH_8B;
  pmodGpsUart.Init.StopBits = UART_STOPBITS_1;
  pmodGpsUart.Init.Parity = UART_PARITY_NONE;
  pmodGpsUart.Init.Mode = UART_MODE_TX_RX;
  pmodGpsUart.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  pmodGpsUart.Init.OverSampling = UART_OVERSAMPLING_16;
  pmodGpsUart.Init.OneBitSampling = UART_ONE_BIT_SAMPLE_DISABLE;

  HAL_UART_Init(&pmodGpsUart);
}

void PmodGPS_HAL_UART_MspInit(UART_HandleTypeDef *huart)
{
  // Enable the clocks for GPIO pins used by the UART3 port (PC4, PC5).
  __HAL_RCC_USART3_CLK_ENABLE();
  __HAL_RCC_GPIOC_CLK_ENABLE();

  GPIO_InitTypeDef GPIO_InitStruct;
  GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
  GPIO_InitStruct.Pull = GPIO_PULLDOWN;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Alternate = GPIO_AF7_USART3;
  GPIO_InitStruct.Pin = GPIO_PIN_4 | GPIO_PIN_5;
  HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);

  // Enable the interrupts generated by USART3 port.
  HAL_NVIC_SetPriority(USART3_IRQn, 1, 0);
  HAL_NVIC_EnableIRQ(USART3_IRQn);
}

void PmodGPS_Write(char* data, int len)
{
  // Non-blocking data transfer.
  HAL_UART_Transmit_IT(&pmodGpsUart, (uint8_t*)data, len);
}

void PmodGPS_Read(uint8_t* data, int len)
{
  // Non-blocking data reception.
  HAL_UART_Receive_IT(&pmodGpsUart,data, len);
}

